/*
 *  Nextview decoder: main module
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation. You find a copy of this
 *  license in the file COPYRIGHT in the root directory of this release.
 *
 *  THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 *  BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
 *  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *
 *  Description: see according C source file.
 *
 *  Author: Tom Zoerner
 *
 *  $Id: epgmain.h,v 1.26 2008/01/22 22:05:16 tom Exp tom $
 */

#ifndef __EPGMAIN_H
#define __EPGMAIN_H


// Interface to Tcl/Tk interpreter for UI modules
#ifdef _TCL
#define TCL_COMM_BUF_SIZE  1000
extern Tcl_Interp *interp;
// add one extra byte at the end of the comm buffer for overflow detection
extern char comm[TCL_COMM_BUF_SIZE + 1];

void AddMainIdleEvent( Tcl_IdleProc *IdleProc, ClientData clientData, bool unique );
bool RemoveMainIdleEvent( Tcl_IdleProc * IdleProc, ClientData clientData, bool matchData );

typedef enum
{
   EPG_ENC_ASCII,       // ASCII (i.e. no transcoding required for UTF-8)
   EPG_ENC_SYSTEM,      // text from current locale (e.g. weekday names) or local files
   EPG_ENC_NXTVEPG,     // text from Nextview EPG source: ISO-8859-1 or UTF-8, depending on compile switch
   EPG_ENC_ISO_8859_1,  // hard-coded ISO-8859-1 (CNI network name table)
} T_EPG_ENCODING;
#define EPG_ENC_NETNAME(AI) ((AI) ? EPG_ENC_NXTVEPG : EPG_ENC_SYSTEM)
extern Tcl_Encoding encIso88591;

Tcl_Obj * TranscodeToUtf8( T_EPG_ENCODING enc,
                           const char * pPrefix, const char * pStr, const char * pPostfix
                           /*,const char * pCallerFile, int callerLine*/ );
//#define TranscodeToUtf8(E,S0,S1,S2) TranscodeToUtf8_Dbg(E,S0,S1,S2,__FILE__,__LINE__)
Tcl_Obj * AppendToUtf8( T_EPG_ENCODING enc, Tcl_Obj * pObj, const char * pStr, const char * pPostfix );

bool EpgMain_StartDaemon( void );
bool EpgMain_StopDaemon( void );
#ifdef WIN32
bool EpgMain_CheckDaemon( void );
#endif
#endif  // _TCL

// Access to databases from UI and acq-ctl
extern EPGDB_CONTEXT * pUiDbContext;

time_t EpgGetUiMinuteTime( void );

#endif  // __EPGMAIN_H
